
//******************************************************************************
//
//   FREQUNCIMETRO DE REDE
//
//   UTILIZA PIC 16F628A E 4 DISPLAY DE 7 SEGUIMENTOS
//   (MTODO POR CAPTURA USANDO O TIMER 1)
//
//   AUTOR : CLAUDIO LARIOS
//
//   data: 09/02/2018
//
//   Uso didtico
//
//   Blog PICSOURCE (LARIOS.TECNOLOGIA.WS)
//
//******************************************************************************

 //  #DEFINE CATODO_COMUM //descomente p/ catodo comum e comente p/ anodo comum
//==============================================================================
#include <16F628a.h>
#fuses HS,NOWDT,NOLVP,NOMCLR
#use delay(clock=4000000)    //tempo por instruo=1us

//bytes
#byte  porta    = 0x05
#byte  portb    = 0x06
#byte  trisa    = 0x85
#byte  trisb    = 0x86
#byte  tmr1h    = 0x0f
#byte  tmr1l    = 0x0e
#byte  ccp1h    = 0x16
#byte  ccp1l    = 0x15
#byte  ccp1con  = 0x17
#byte  t1con    = 0x10

//bits
#bit  gie       = 0x0b.7
#bit  peie      = 0x0b.6
#bit  t0ie      = 0x0b.5
#bit  ccp1ie    = 0x8c.2
#bit  ccp1if    = 0x0c.2
#bit  tmr1_on   = 0x10.0
#bit  tmr1if    = 0x0c.0
#bit  in_rede   = 0x06.3

//variavis globais
unsigned int8  dig3,dig2,dig1,dig0,mux,xdig3,xdig2,xdig1,xdig0,n_amostra=0;
unsigned int1 flag_n=0;
unsigned int16 timeout=0,res0,res1;
unsigned int32 somatoria=0;
// Tabela para retorno dos seguimentos correspondentes aos nmeros 0-9
byte const tabela [] = { 0x40,0xf1,0x24,0x30,0x91,0x12,0x02,0xf0,0x0,0x10};

/*******************************************************************************
   ROTINA DE INTERRUPO POR CAPTURA NO TIMER 1
*******************************************************************************/
#int_ccp1
void isr_capture_rb3(){
 unsigned int32 res=0;
      timeout=0;//zera timeout e impede zeramento dos displays
      if(!flag_n){//primeira leitura
      res0=ccp1l|(int16)ccp1h<<8;//pega valor da primeira leitura
      flag_n=1;//seta flag_n
      }
      else{//segunda leitura
      res1=ccp1l|(int16)ccp1h<<8;//pega valor da segunda leitura
      flag_n=0;// reseta flag_n
      if(res1>=res0){res=res1-res0;}//subtrai a 1 leitura da 2 leitura
      else {
      res=res1-res0+0xffff;//caso 2 leitura seja menor
      }
      somatoria+=res;//soma 30 resultados para obter uma mdia da frequncia
      n_amostra++;//incrementa
      if(n_amostra!=30)return;//sai se no atingiu ainda 30 resultados

      n_amostra=0;//zera contador
      res=somatoria/30;//mdia de 30 amostragens
      res= 100000000/res;//ajusta para trabalhar com inteiros
      if(res>9999)return;//se leitura for fora de escala, esta ser desprezada
      xdig3=0;//zera
      while(res>=1000){ xdig3++;res=res-1000; }//obtem milhares
      xdig2=0;//zera
      while(res>=100){ xdig2++;res=res-100; }//obtm centenas
      xdig1=0;//zera
      while(res>=10){ xdig1++; res=res-10;}//obtm dezenas
      xdig0=res;//obtm unidades
      somatoria=0;//zera somatoria
    }
}

//==============================================================================
//          ROTINA DE MULTIPLEXAO DOS DISPLAYS
//==============================================================================

//multiplexa display de 7 seg
 void multiplexa() {
  if(++mux>3){  mux=0; dig0=xdig0; dig1=xdig1; dig2=xdig2; dig3=xdig3;  }
  porta=0; //apaga seguimentos para mudana

  #IFDEF  CATODO COMUM
  portb=0x00;
  porta=0;
  #ELSE
  porta=0x10;
  portb=0xff;
  #ENDIF

  switch (mux) {
  case 0:
   portb=tabela[dig0];
   #IFDEF  CATODO_COMUM
   portb^=0x7f;
   #ENDIF
   porta|=1;
   break;
  case 1:
   portb=tabela[dig1];
   #IFDEF  CATODO_COMUM
   portb^=0x7f;
   #ENDIF
   porta|=2;
   break;

  case 2:
   portb=tabela[dig2];
   #IFDEF  CATODO_COMUM
   porta|=0x10; //liga ponto
   #else
    porta&=0xef; //liga ponto
   #endif

   #IFDEF  CATODO_COMUM
   portb^=0x7f;
   #ENDIF
   porta|=4;
   break;

  case 3:
   if (dig3){//apaga digito da dezena caso for '0'
   portb=tabela[dig3];
   #IFDEF  CATODO_COMUM
   portb^=0x7f;
   #ENDIF
   porta|=8;
   break;
   }
  }
   delay_ms(2);//delay de apresentao do dgito (p/ freq ~500 Hertz)
}
//==============================================================================
//           ROTINA DE TIMEOUT
//==============================================================================

 void time_out(){//zero os dgitos se ficar sem sinal na entrada
 #DEFINE TEMPO_MAX_SEM_SINAL  800
   timeout++;
   if(timeout>=TEMPO_MAX_SEM_SINAL){
    xdig3=0;xdig2=0;xdig1=0;xdig0=0;
    timeout=0;
   }
 }

//==============================================================================
//           ROTINA PRINCIPAL
//==============================================================================

void main() {

   setup_comparator(NC_NC_NC_NC);//desliga comparadores internos
   #IFDEF CATODO_COMUM
   portb&=0x80;
   #ELSE
   portb|=0X7F;
   #ENDIF
   porta=0;
   trisb= 0b00001000;  //sentido dos pinos do port B
   trisa= 0b11100000;//sentido dos pinos do port A
   dig3=0; dig2=0;dig1=0;dig0=0;
   xdig3=0; xdig2=0;xdig1=0;xdig0=0;
   delay_ms(50);
   t1con=0x05;
   tmr1_on=1;//liga timer 1
   ccp1con=0x05;//capture on 1 rising
   ccp1if=0;//reseta flag do capture
   peie=1;//liga interupo de perifricos
   ccp1ie=1;//liga interrupo do capture
   gie=1; //habilita interrupes geral

//==============================================================================
//           LOOP  PRINCIPAL
//==============================================================================
   for(;;) {
      multiplexa();//multiplexa displays
      time_out();//verifica existncia de sinal na entrada, seno zera displays
  }//while
}//main
